<?php
// Autor (c) Miroslav Novak, www.platiti.cz
// Pouzivani bez souhlasu autora neni povoleno
// #Ver:PRV098-49-ga9cb64a5:2022-11-14#

use UniModul\Base\SessionInterface;
use UniModul\Base\StandardSession;

require_once('DatabaseConnection.php');
require_once('UniStructures.php');
require_once('UniDebug.php');
require_once('UniSync.php');
require_once('Session.php');

class UniModulConfig extends CheckNonexistentFieldsLogOnly {
	/** @var ?string */
	var $mysql_server;
	/** @var ?string */
	var $mysql_dbname;
	/** @var ?string */
	var $mysql_login;
	/** @var ?string */
	var $mysql_password;
	/** @var ?IDatabaseConnection */
	var $databaseConnection;  // IDatabaseConnection, if it is set, the field mysql_* are not used
	/** @var string */
	var $uniModulDirUrl;
	/** @var callable(string, array<string, string>) : string */
	var $funcGetCallbackUrl;
	/** @var callable(OrderReplyStatus) : void */
	var $funcProcessReplyStatus; //urceno typicky pro davkove parovani transakci z banky atp., jinak pouzit standardni reply a notification
	/** @var string */
	var $adapterName;
	/** @var SessionInterface */
	public $session;
}

abstract class UniModul extends CheckNonexistentFieldsLogOnly {
	/** @var string */
	var $name;
	/** @var UniLogger */
	var $logger;
	/** @var UniModulConfig */
	var $baseConfig;
	/** @var object */
	public $config; //public kvuli mocku
	/** @var UniDictionary */
	public $dictionary;
	/** @var ?string */
	public $subMethod = null; // 1 - pokud adapter dokaze v jednom modulu zobrazit vice metod, jmeno submetody je-li pro adapter znama jiz pri konstrukci, null nebo "" pro defaultni modul
	/** @var IDatabaseConnection */
	protected $dbConn;
	/** @var array<string, mixed> */
	public $uniModulProperties = array(); // asoc pole vlastnosti/pozadavku UniModulu
	/** @var string */
	public $version;
	/** @var string */
	public $versionDate;
	/** @var string */
	public $versionStr;
	/** @var string */
	var $configAcke;
	/** @var bool */
	var $activationKeyOk;


	/**
	 * @param string $name
	 * @param ConfigSetting $configSetting
	 * @param ?string $subMethod
	 * @return void
	 */
	public function __construct($name, $configSetting, $subMethod) {
		$this->name = $name;
		$this->subMethod = $subMethod;
		if (!$this->versionStr) $this->versionStr = "#Ver:PRV098-49-ga9cb64a5:2022-11-14#";
		preg_match("/^#Ver:(.+):(.+)#/", $this->versionStr, $verpars);
		$this->version = isset($verpars[1]) ? $verpars[1] : 'DEVV';
		$this->versionDate = isset($verpars[2]) ? $verpars[2] : 'DEVT';
		$this->logger = new UniLogger();
		if ($configSetting!==null) {
			$this->baseConfig = $configSetting->uniModulConfig;
			if ($this->baseConfig->databaseConnection != null) {
				$this->dbConn = $this->baseConfig->databaseConnection;
			} else if ($this->baseConfig->mysql_server !== null && $this->baseConfig->mysql_login  !== null && $this->baseConfig->mysql_dbname !== null) {
				$this->dbConn = new MysqliConnectionShared($this->baseConfig->mysql_server, $this->baseConfig->mysql_login, (string)$this->baseConfig->mysql_password, $this->baseConfig->mysql_dbname);
			} else if ($this->baseConfig->mysql_server !== null || $this->baseConfig->mysql_login  !== null || $this->baseConfig->mysql_dbname !== null) {
				throw new Exception("Missing database connection parameters");
			} // else connection not created, various get config info etc
			$this->checkConfig($configSetting);
		}
		$this->dictionary = $this->getDictionary();
	}

	/** @return UniDictionary  */
	protected function getDictionary() {
		return new UniDictionary($this->name, $this->logger);
	}


	/**
	 * @param string $language
	 * @return ConfigInfo
	 */
	abstract public function getConfigInfo($language='en'); //vola to jen UniFactory
	//vyreseno, neni nutne soucast interfacu//  abstract public function setConfigFromData($configSetting);  //vola to konstruktor odvozeneho UniModulu, nekdy ale kdyz konfigurace neni k mani pri konstrukci adapteru, tak se to volat jeste dodatecne

	// vrati mozne subMetody formou Array of strings
	/** @return array<int, string>  */
	public function getSubMethods() {
		return array();
	}


	//// detail produktu na splatky
	/**
	 * @param ?string $shopBaseUrl - deprecated, use baseConfig->uniModulDirUrl or getModulSubMethodLogoImage2
	 * @param string $currency
	 * @param float $price
	 * @param string $language
	 * @return ?string
	 */
	public function ProductGetInstallmentEmbedHtml($shopBaseUrl, $currency, $price, $language='cs') {
		return null;
	}

	//// volane pri placeni
	/**
	 * @param OrderToPayInfo $orderToPayInfo
	 * @return PrePayGWInfo
	 */
	abstract public function queryPrePayGWInfo($orderToPayInfo);
	/**
	 * @param OrderToPayInfo $orderToPayInfo
	 * @return RedirectAction
	 */
	abstract public function gatewayOrderRedirectAction($orderToPayInfo);
	/**
	 * @param string $language
	 * @return OrderReplyStatus
	 */
	abstract public function gatewayReceiveReply($language='en');

	/**
	 * @return OrderReplyStatus
	 */
	public function gatewayReceiveNotification() {
		throw new Exception('Not implemented');
	}


	/**
	 * @param OrderReplyStatus $orderReplyStatus
	 * @param string $shopOrderNumber
	 * @return void
	 */
	public function updateShopOrderNumber($orderReplyStatus, $shopOrderNumber) {
		$sql = "update unimodul_transactions set dateModified = FROM_UNIXTIME(".time()."), shopOrderNumber=".toSql($shopOrderNumber)." where gwOrderNumber = ".toSql($orderReplyStatus->gwOrderNumber)." and uniModulName=".toSql($this->name);
		$this->dbConn->sqlExecute($sql);
	}




	/**
	 * @param string $gwOrderNumber
	 * @param int $transactionPK
	 * @return null|TransactionRecord
	 */
	public function getOrderTransactionRecordFromDbUnique($gwOrderNumber, $transactionPK=null) {
		return $this->getOrderTransactionRecordFromDbPriv(null, $gwOrderNumber, null, $transactionPK);
	}

	// pri prohlizeni obejdnavky napr v prehledu objednavek v eshopu pro zjisteni napr. c.obj. na brane
	/**
	 * @param ?string $shopOrderNumber
	 * @param ?string $shopPairingInfo
	 * @return null|TransactionRecord
	 */
	public function getOrderTransactionRecordFromDbLast($shopOrderNumber, $shopPairingInfo=null) {
		return $this->getOrderTransactionRecordFromDbPriv($shopOrderNumber, null, $shopPairingInfo, null);
	}

	// tato funkce nebude volana niky primo ale jen pres ty dve funkce vyse, protoze micha unikatni selecty s neunikatnim poslednima
	/**
	 * @param string $shopOrderNumber
	 * @param string $gwOrderNumber
	 * @param string $shopPairingInfo
	 * @param int $transactionPK
	 * @return null|TransactionRecord
	 */
	public function getOrderTransactionRecordFromDb($shopOrderNumber, $gwOrderNumber=null, $shopPairingInfo=null, $transactionPK=null) {
		UniWriteErrLog(0, "MyWARN: Zastarale, pouzij getOrderTransactionRecordFromDbUnique nebo getOrderTransactionRecordFromDbLast", __FILE__, __LINE__ );
		return $this->getOrderTransactionRecordFromDbPriv($shopOrderNumber, $gwOrderNumber, $shopPairingInfo, $transactionPK);
	}

	/**
	 * @param ?string $shopOrderNumber
	 * @param ?string $gwOrderNumber
	 * @param ?string $shopPairingInfo
	 * @param ?int $transactionPK
	 * @return null|TransactionRecord
	 */
	private function getOrderTransactionRecordFromDbPriv($shopOrderNumber, $gwOrderNumber=null, $shopPairingInfo=null, $transactionPK=null) {
		if (!is_null($transactionPK)) {
			$where = "transactionPK=".toSql($transactionPK);
		} else if (!is_null($shopOrderNumber)) {
			$where = "shopOrderNumber=".toSql($shopOrderNumber);
		} else if (!is_null($gwOrderNumber)) {
			$where = "gwOrderNumber=".toSql($gwOrderNumber);
		} else if (!is_null($shopPairingInfo)) {
			$where = "shopPairingInfo=".toSql($shopPairingInfo);
		} else {
			throw new Exception("At least one identifiear has to be provided");
		}
		$sql = "select * from unimodul_transactions where ".$where." and uniModulName=".toSql($this->name). " order by if(orderStatus=0, 99, orderStatus), transactionPK desc";
		$ar = $this->dbConn->sqlQuery($sql);

		if (count($ar)==0) { // to je potreba napr pro Prestu, kde v adminu hookovane infa o gwOrdnumberu se volaji vsechny moduly, i kdyz nemaji nic spolecneho s danou platbou
			return null;
		}
		if (count($ar)>1 && ($gwOrderNumber !== null || $transactionPK !== null)) {
			user_error("duplikatni zaznamy v unimodul_transaction: ".$sql);
		}
		return $this->loadTransactionRecordFromDbRow($ar[0]);
	}


	/**
	 * @param array<string, mixed> $row
	 * @return TransactionRecord
	 */
	protected function loadTransactionRecordFromDbRow($row) {
		$transactionRecord = new TransactionRecord();
		$transactionRecord->transactionPK =intval($row['transactionPK']);
		$transactionRecord->uniModulName = strval($row['uniModulName']);
		$transactionRecord->gwOrderNumber = strvalNull($row['gwOrderNumber']);
		$transactionRecord->gwPairingInfo = strvalNull($row['gwPairingInfo']);
		$transactionRecord->shopOrderNumber = strvalNull($row['shopOrderNumber']);
		$transactionRecord->shopPairingInfo = strval($row['shopPairingInfo']);
		$transactionRecord->uniAdapterData = unserialize(strval($row['uniAdapterData']));
		$transactionRecord->uniModulData = unserialize(strval($row['uniModulData']));
		$transactionRecord->forexNote = strvalNull($row['forexNote']);
		$transactionRecord->orderStatus = intval($row['orderStatus']);
		$transactionRecord->dateCreated = new DateTime(strval($row['dateCreated']));
		$transactionRecord->dateModified = $row['dateModified'] === null ? null : new DateTime(strval($row['dateModified']));
		return $transactionRecord;
	}

	// rozhrani pro zdedene UniModuly  - protected

	/**
	 * @param ?string $shopOrderNumber
	 * @param string $shopPairingInfo
	 * @param ?string $gwOrderNumber
	 * @param ?string $forexNote
	 * @param int $orderStatus
	 * @param mixed $uniAdapterData
	 * @param mixed $uniModulData
	 * @return int
	 */
	protected function writeOrderToDb($shopOrderNumber, $shopPairingInfo, $gwOrderNumber, $forexNote, $orderStatus=null, $uniAdapterData=null, $uniModulData=null) {
		if ($orderStatus === null) {
			$orderStatus = OrderStatus::$initiated;
		}
		$sql = "insert into unimodul_transactions (uniModulName, gwOrderNumber, shopOrderNumber, shopPairingInfo, forexNote, orderStatus, uniAdapterData, uniModulData, dateCreated) values ("
		.toSql($this->name).", "
		.toSql($gwOrderNumber).","
		.toSql($shopOrderNumber).","
		.toSql($shopPairingInfo).","
		.toSql($forexNote).","
		.$orderStatus.","
		.toSql(serialize($uniAdapterData)).","
		.toSql(serialize($uniModulData)).","
		."FROM_UNIXTIME(".time().")"
		.")";
		$this->dbConn->sqlExecute($sql);
		$insId = $this->dbConn->getInsertId();
		return $insId;
	}


	/**
	 * @param int $transactionPK
	 * @param string $gwOrderNumber
	 * @param string $gwPairingInfo
	 * @return void
	 */
	protected function updateGwOrderNumber($transactionPK, $gwOrderNumber, $gwPairingInfo = null) {
		if ($gwPairingInfo !== null) {
			$pinf = ", gwPairingInfo=".toSql($gwPairingInfo) . " ";
		} else {
			$pinf = "";
		}
		$sql = "update unimodul_transactions set dateModified = FROM_UNIXTIME(".time()."), gwOrderNumber=".toSql($gwOrderNumber) . $pinf . " where transactionPK = ".toSql($transactionPK);
		$this->dbConn->sqlExecute($sql);
	}

	/**
	 * @param OrderReplyStatus $orderReplyStatus
	 * @param ?int $transactionPK
	 * @return void
	 */
	protected function updateOrderReplyStatusGwOrdNumInDb($orderReplyStatus, $transactionPK = null) {
		if ($transactionPK !== null) {
			$sql = "update unimodul_transactions set DateModified = FROM_UNIXTIME(".time()."), orderStatus=".toSql($orderReplyStatus->orderStatus).", gwOrderNumber=".toSql($orderReplyStatus->gwOrderNumber).", gwPairingInfo=".toSql($orderReplyStatus->gwPairingInfo)." where transactionPK = ".toSql($transactionPK);
			$this->dbConn->sqlExecute($sql);
		} else if ($orderReplyStatus->gwOrderNumber != null) {
			$sql = "update unimodul_transactions set DateModified = FROM_UNIXTIME(".time()."), orderStatus=".toSql($orderReplyStatus->orderStatus).", gwPairingInfo=".toSql($orderReplyStatus->gwPairingInfo)." where gwOrderNumber = ".toSql($orderReplyStatus->gwOrderNumber)." and uniModulName=".toSql($this->name);
			$this->dbConn->sqlExecute($sql);
		} else {
			user_error("updateOrderReplyStatusGwOrdNumInDb chybi transactionId i orderReplyStatus->gwOrderNumber");
		}
	}

	/**
	 * @param OrderReplyStatus $orderReplyStatus
	 * @return int
	 */
	protected function ensureGlobalPairingInfoStatusUpgradeOnly($orderReplyStatus) {
		// overime ze nedojde ke snizeni urovne dokonceni objednavky podle pairingInfo, aby se nezmenil stav v eshopu po te co nektery z predchozich pokusu o zaplaceni zpozdene zahlasi neuspech
		if ($orderReplyStatus->orderStatus == OrderStatus::$successful || $orderReplyStatus->orderStatus == OrderStatus::$invalidReply) {
			$hledStat = null;
		} else if ($orderReplyStatus->orderStatus == OrderStatus::$pending) {
			$hledStat = OrderStatus::$successful;
		} else {
			$hledStat = OrderStatus::$successful . ',' . OrderStatus::$pending;
		}
		if ($hledStat !== null) {
			$sql = "select * from unimodul_transactions where orderStatus in (".$hledStat.") and shopPairingInfo=".toSql($orderReplyStatus->shopPairingInfo) . " and gwOrderNumber != ".toSql($orderReplyStatus->gwOrderNumber);
			$ar = $this->dbConn->sqlQuery($sql);
		} else {
			$ar = array();
		}
		if (count($ar) != 0) {
			$this->logger->writeLog("WARNING: Jiz existuje stav platba objednavky s vyssi urovni zaplaceni, prepiname na invalidReply. Status stavajici objednavky je ".$orderReplyStatus->orderStatus.", ale existujici obejdnavka GwOrdNum=" . $ar[0]['gwOrderNumber'] . " ma stav " . $ar[0]['orderStatus']);
			return OrderStatus::$invalidReply;
		} else {
			return $orderReplyStatus->orderStatus;
		}
	}

	/**
	 * @param int $transactionPK
	 * @param mixed $uniAdapterData
	 * @return void
	 */
	public function updateUniAdapterDataInDb($transactionPK, $uniAdapterData) {
		$sql = "update unimodul_transactions set dateModified = FROM_UNIXTIME(".time()."), uniAdapterData=".toSql(serialize($uniAdapterData))." where transactionPK = ".toSql($transactionPK);
		$this->dbConn->sqlExecute($sql);
	}

	/**
	 * @param int $transactionPK
	 * @param mixed $uniModulData
	 * @return void
	 */
	public function updateUniModulDataInDb($transactionPK, $uniModulData) {
		$sql = "update unimodul_transactions set dateModified = FROM_UNIXTIME(".time()."), uniModulData=".toSql(serialize($uniModulData))." where transactionPK = ".toSql($transactionPK);
		$this->dbConn->sqlExecute($sql);
	}


	// zjisteni pending transakci pri offline zjistovani stavu transakce z cronu
	/**
	 * @param ?array<int, int> $statuses
	 * @param ?DateTimeInterface $since
	 * @return array<TransactionRecord>
	 */
	protected function getAllPendingOrderTransactionRecords($statuses = null, $since = null) {
		if ($statuses === null) $statuses = array(OrderStatus::$pending);
		array_walk($statuses, function (&$s) {$s = toSql($s);});
		$dateCond = '';
		if ($since !== null) {
			$dateCond = ' and dateCreated >= ' . toSql(date_format($since,"Y-m-d H:i:s"));
		}
		$sql = "select * from unimodul_transactions where uniModulName=".toSql($this->name)." and orderStatus in (".implode(",", $statuses).") " . $dateCond . "order by greatest(dateCreated, dateModified)";
		$ar = $this->dbConn->sqlQuery($sql);
		$pendingTransactions = array();
		foreach($ar as $ar0) {
			$transactionRecord = $this->loadTransactionRecordFromDbRow($ar0);
			$pendingTransactions[]=$transactionRecord;
		}
		return $pendingTransactions;
	}

	/**
	 * @param OrderToPayInfo $orderToPayInfo
	 * @param ?bool $currencySupported
	 * @return array{bool, string, number, ?string, ?string, ?OrderReplyStatus}
	 */
	protected function fixCurrency($orderToPayInfo, $currencySupported=null) {
		global $vrfacke;
		if ($currencySupported===null && isset($this->config->supportedCurrencies)) {
			$currencySupported = empty($orderToPayInfo->currency) ? false : strpos($this->config->supportedCurrencies, $orderToPayInfo->currency)!==false;
		}
		$forexMessage = null;
		$forexNote = null;
		$newtotal = $orderToPayInfo->amount;
		$actcur = $orderToPayInfo->currency;
		$newcur = $actcur;
		$newtotal = $orderToPayInfo->amount;
		$orderReplyStatusFail = null;
		if ($newtotal == 0) {
			$isPossible = false;
			$orderReplyStatusFail = $this->createImmediateReplyStatus($orderToPayInfo, OrderStatus::$failedRetriable, "Zero amount");
		} else if (!$currencySupported) {
			if (!empty($this->config->convertToCurrencyIfUnsupported) && $orderToPayInfo->currencyRates !== null) {
				$newcur = strval($this->config->convertToCurrencyIfUnsupported);
				$rate = $orderToPayInfo->currencyRates[$newcur] / $orderToPayInfo->currencyRates[$actcur];
				$newtotal = $orderToPayInfo->amount * $rate;

				$orderToPayInfo->amount = $newtotal;
				$orderToPayInfo->currency = $newcur;
				if ($orderToPayInfo->cartItems != null) {
					array_walk($orderToPayInfo->cartItems, function($v, $k) use ($rate) {$v->unitPrice *= $rate;});
				}

				$ratestr = number_format($rate, 4, '.', ' ');
				$newtotalstr = number_format($newtotal, 2, '.', ' ');

				$forexMessageTemplate = $this->dictionary->get('forexMessageTemplate', $orderToPayInfo->language);
				$forexMessage = strtr($forexMessageTemplate, array(
					"{actcur}"=>$actcur,
					"{newcur}"=>$newcur,
					"{newtotalstr}"=>$newtotalstr,
					"{ratestr}"=>$ratestr,
				));

				$forexNote = "{$orderToPayInfo->amount} $actcur -> $newtotalstr $newcur @ $ratestr $newcur/$actcur";
				if (!empty($GLOBALS['UniModul_TrackDisplayPaymentOption'])) {
					$this->logger->writeLog("fixCurrency " . $forexNote);
				}
				$isPossible = true;
			} else {
				$isPossible = false;

				$resultText = $this->dictionary->get('unsupportedCurrency', $orderToPayInfo->language)." ".$orderToPayInfo->currency;
				$orderReplyStatusFail = $this->createImmediateReplyStatus($orderToPayInfo, OrderStatus::$failedRetriable, $resultText);
			}
		} else {
			$isPossible = true;
		}
		$ok = $vrfacke($this);
		$isPossible = $isPossible && $ok;

		if (!empty($GLOBALS['UniModul_TrackDisplayPaymentOption'])) {
			$this->logger->writeLogNoNewLines("fixCurrency (".$this->name." ".$this->subMethod."): isPossible = " . ($isPossible?"true":"false") . "   currencySupported:".($currencySupported?"true":"false")  . "  orderToPayInfo:".print_r($orderToPayInfo, true) . "  " . ($ok?"ACK":" NAK"));
		}
		return array($isPossible, $newcur, $newtotal, $forexMessage, $forexNote, $orderReplyStatusFail);
	}

	/**
	 * @param OrderToPayInfo $orderToPayInfo
	 * @param int $orderStatus
	 * @param ?string $resultText
	 * @param ?string $successHtml
	 * @return OrderReplyStatus
	 */
	function createImmediateReplyStatus($orderToPayInfo, $orderStatus, $resultText = null, $successHtml = null) {
		$orderReplyStatusFail = new OrderReplyStatus();
		$orderReplyStatusFail->orderStatus = $orderStatus;
		$orderReplyStatusFail->resultText = $resultText;
		$orderReplyStatusFail->successHtml = $successHtml;
		$orderReplyStatusFail->shopOrderNumber = $orderToPayInfo->shopOrderNumber;
		//$orderReplyStatusFail->gwOrderNumber = $gwOrderNumber;
		$orderReplyStatusFail->shopPairingInfo = $orderToPayInfo->shopPairingInfo;
		$orderReplyStatusFail->uniAdapterData = $orderToPayInfo->uniAdapterData;
		return $orderReplyStatusFail;
	}

	/**
	 * @param OrderToPayInfo $orderToPayInfo
	 * @param int $orderStatus
	 * @param ?string $resultText
	 * @param ?string $successHtml
	 * @return RedirectAction
	 */
	public function getImmediateReplyStatusRedirectAction($orderToPayInfo, $orderStatus, $resultText = null, $successHtml = null) {
		$redirectAction = new RedirectAction();
		$redirectAction->orderReplyStatus = $this->createImmediateReplyStatus($orderToPayInfo, $orderStatus, $resultText, $successHtml);
		return $redirectAction;
	}

	// infoBox

	/**
	 * @param string $uniAdapterName
	 * @param string $language
	 * @return InfoBoxData
	 */
	public function getInfoBoxData($uniAdapterName, $language) {
		$infoBoxLinkTextTemplate = $this->dictionary->get('infoBoxPlatitiLinkText', $language);
		$linkText = strtr($infoBoxLinkTextTemplate, array(
			"{modulname}"=>$this->name,
			"{shopname}"=>$uniAdapterName,
		));

		$infoBoxData = new InfoBoxData();
		$infoBoxData->title = $this->name;
		$infoBoxData->image = "Uni".$this->name."Logo.png";
		//$infoBoxData->link = "http://www.payu.cz";  // dosadi se az v konkretnim modulu
		$infoBoxData->platitiLink = "http://www.platiti.cz/{$uniAdapterName}-{$this->name}.php";
		$infoBoxData->platitiLinkText = $linkText;
		return $infoBoxData;
	}

	// helpers

	/**
	 * @param string $oldUrl
	 * @param array<string, string> $argsArray
	 * @return string
	 */
	public function appendUrlQueryArgs($oldUrl, $argsArray) {
		return $oldUrl . (strpos($oldUrl, '?')===false ? '?' : '&') . http_build_query($argsArray);
	}

	/**
	 * @param string $form
	 * @return never
	 */
	public function formRedirect($form) {
		//ob_clean();
		echo '<html><head><meta http-equiv="Content-Type" content="text/html; charset=utf-8"/></head><body>';
		echo $form;
		echo "</body></html>";
		//ob_flush();
		//flush();
		ResetUniErr();
		exit();
	}

	/**
	 * @param string $redirUrl
	 * @return never
	 */
	public function jsRedirect($redirUrl) {
		$script = "<script>setTimeout(function() { window.location.href='".$redirUrl."';}, 200);</script><noscript><a href='".$redirUrl."'>Klikněte prosím/ Please click</a></noscript>";
		$this->formRedirect($script);
	}

	/**
	 * @param ConfigSetting $configSetting
	 * @return void
	 */
	function checkConfig($configSetting) {
		if ($configSetting != null && $configSetting->configData != null) {
			$this->configAcke = strval($configSetting->configData['activationKey']);
			$chksumok = true;
			foreach(explode(' ',$this->configAcke) as $akey0) {
				$akey = str_replace('-', '', $akey0);
				if (strlen($akey)<2) continue;
				if (substr(md5(substr($akey,0,-2)), 0,2) != substr($akey,-2)) {
					$chksumok = false;
					$this->logger->writeLog("Wro"."ng key " . $akey0);
				}
			}
			$this->activationKeyOk = $chksumok;

			if (!function_exists('vrfacke')) {
				$GLOBALS['vrfacke'] = 'vrfacke';
				function vrfacke ($ths) { /** @phpstan-ignore-line */
					$dom = $GLOBALS["_SER"."VER"]["HTT"."P_H"."OST"];
					$actKeys = $ths->configAcke;
					$mod = $ths->name;
					$ada = $ths->baseConfig->adapterName;
					$upddatee = explode('-', $ths->versionDate);
					$upddate15 = count($upddatee)>2 ? (intval($upddatee[0])-2015)*12+(intval($upddatee[1])-1) : 0;

					$ok = false;
					/** @var string */
					$dom = preg_replace('/(:[0-9]+)/', '', $dom);
					/** @var string */
					$dom3 = preg_replace('/(^www\.)/', '', $dom);
					if ($dom3=='localhost' || $dom3=='127.0.0.1') {
						$ok = true;
					}
					foreach(explode(' ',$actKeys) as $actkey) {
						$actkey = str_replace('-', '', $actkey);
						/** @var string */
						$actkey = preg_replace('/[^0-9a-f]/','0',$actkey);
						$actkey = str_pad($actkey, 14, '0');

						$rndh = substr($actkey, 0, 2);
						$datesh = substr($actkey, 2, 6);
						$dates = intval(base_convert($datesh, 16, 10)) ^ intval(base_convert($rndh.$rndh.$rndh, 16, 10));
						$maxdate = intval($dates>>12);
						$maxupddate = $dates % (1<<12);
						$date15 = (date("y")-15)*12+(date("m")-1);
						$dateok = $date15<=$maxdate && $upddate15 <= $maxupddate;

						$numhash = intval(base_convert(substr(md5($dom3.$ada.$mod.$datesh.$rndh),0, 4), 16, 10)) % 61423;
						$numhashdev = intval(base_convert(substr(md5($dom3.$datesh.$rndh),0, 4), 16, 10)) % 61423;
						$dec = bcpowmod(base_convert(substr($actkey,8,4), 16, 10), strval(43), strval(61423), 0);
						$domok = $numhash == $dec || $numhashdev == $dec;

						$ok = $ok || ($dateok && $domok);
					}
					if (!$ok) {
						$ths->logger->writeLog("Ba"."d key $ada $mod $dom '$actKeys'");
					}
					return $ok;
				}
			}

		}
	}

	/**
	 * @param string $language
	 * @param ?string $subMethod
	 * @return string
	 */
	public function getModuleSubMethodName($language='en', $subMethod='') {
		if ($subMethod == '') $subMethod = $this->subMethod;
		if ($subMethod == '') {
			$subname = $this->dictionary->get('payment_method_name', $language);
		} else {
			$subname = $this->dictionary->get('submethod_name_'.$subMethod, $language);
		}
		return $subname;
	}

	/**
	 * @param string $unused_shopBaseUrl
	 * @param string $subMethod
	 * @return string
	 */
	public function getModulSubMethodLogoImage($unused_shopBaseUrl, $subMethod='') {
		// DEPRECATED, use getModulSubMethodLogoImage2 instead
		return $this->getModulSubMethodLogoImage2($subMethod='');
	}

	/**
	 * @param ?string $subMethod
	 * @return string
	 */
	public function getModulSubMethodLogoImage2($subMethod='') {
		if ($subMethod == '') $subMethod = $this->subMethod;
	    if (file_exists(dirname(__FILE__)."/Uni".$this->name.$subMethod."Logo.png")) {
	    	$submetLogoPiece = $subMethod;
	    } else {
	    	$submetLogoPiece = "";
	    }
	    $logoimg = $this->baseConfig->uniModulDirUrl . "Uni".urlencode($this->name . $submetLogoPiece)."Logo.png";
	    return $logoimg;
	}


	/** @return string  */
	public function getMiniLogoUrl() {
		return "https://www.platiti.cz/muzo/minilogo.png";
	}

	/** @return string  */
	public function getMiniLogoSpan() {
		return "<span style=\"background:url('".$this->getMiniLogoUrl()."')\"></span>";
	}

	/**
	 * @param string $callbackName
	 * @param array<string, string> $arguments
	 * @return void
	 */
	public function processCallbackRequest($callbackName, $arguments) {
		$this->logger->writeLog("Cannot process callback request " . $_SERVER['REQUEST_URI']);
		user_error("Cannot process callback request");
	}

	// dopocita nepovinne polozky v cartItems
	/**
	 * @param OrderToPayInfo $orderToPayInfo
	 * @return void
	 */
	public function normalizeCartItems($orderToPayInfo /*updatuje*/) {
		$vychoziOrderToPayInfo = unserialize(serialize($orderToPayInfo));
		foreach($orderToPayInfo->cartItems as $item) {
			if (empty($item->type)) {
				$itemName = (string)$item->name;
				if ($itemName == 'shipping / doprava') $item->type = UniCartItemType::delivery;
				else if ($itemName == 'discount / sleva') $item->type = UniCartItemType::discount;
				else $item->type = UniCartItemType::commodity;
			}
			if ($item->unitTaxAmount === null) {
				$item->unitTaxAmount = $item->unitPrice * (1 - 100/(100 + $item->taxRate));
			}
		}
	}

	// sesumiruje polozkyp pres jednotlive sazby a provede korekci zaokrouhleni, vysledek EetCastkySazby
	/**
	 * @param OrderToPayInfo $orderToPayInfo
	 * @param bool $convertToCzk
	 * @return null|EetCastkySazby
	 */
	public function getEetRozdeleni($orderToPayInfo, $convertToCzk = true) {

		if (empty($orderToPayInfo->cartItems)) {
			$this->logger->writeLog("Prazdne cart Items");
			return null;
		}

		// secteni pres sazby
		$rozdeleni = array();
		$rozdeleniTax = array();
		foreach ($orderToPayInfo->cartItems as $ci) {

			$act = $ci->unitPrice * $ci->quantity;
			$cum = isset($rozdeleni[$ci->taxRate]) ? $rozdeleni[$ci->taxRate] : 0;
			$rozdeleni[($ci->taxRate)] = $cum + $act;

			if ($ci->unitTaxAmount !== null) {
				$act = $ci->unitTaxAmount * $ci->quantity;
			} else {
				$koef = $ci->taxRate/(100 + $ci->taxRate);
				$act = $ci->unitPrice * $koef  * $ci->quantity;
			}
			$cum = isset($rozdeleniTax[$ci->taxRate]) ? $rozdeleniTax[$ci->taxRate] : 0;
			$rozdeleniTax[($ci->taxRate)] = $cum + $act;

		}

		// prevod do CZK
		if ($convertToCzk && $orderToPayInfo->currency != 'CZK' && isset($orderToPayInfo->currencyRates['CZK'])) {
			$fxrate = $orderToPayInfo->currencyRates['CZK'] / $orderToPayInfo->currencyRates[$orderToPayInfo->currency];
			array_walk($rozdeleni, function(&$v, $k) use ($fxrate) {$v = $v * $fxrate;});
			array_walk($rozdeleniTax, function(&$v, $k) use ($fxrate) {$v = $v * $fxrate;});
			$total =$orderToPayInfo->amount * $fxrate;
		} else {
			$total =$orderToPayInfo->amount;
		}


		// zaokrouhleni na halere
		array_walk($rozdeleni, function(&$v, $k) {$v = round($v, 2);});
		array_walk($rozdeleniTax, function(&$v, $k) {$v = round($v, 2);});

		// korekce pro pripadne zaokrouhleni do nejnizsi ze sazeb
		$total = round($total, 2);
		$vychoziSuma = array_sum($rozdeleni);
		$rozdilZaokrouhleni = $total - $vychoziSuma;
		$minsazba = array_reduce(array_keys($rozdeleni), 'min', 999999);
		$rozdeleni[$minsazba] += $total - $vychoziSuma;
		if (abs($rozdilZaokrouhleni) > 10) {
			$this->logger->writeLog("Nesouhlasi castky v polozkach kosiku. Placena castka " . $total . ", soucet polozek je " . $vychoziSuma . "Polozky kosiku: " . print_r($orderToPayInfo->cartItems, true));
			return null;
		}

		// kontrola na pripustne sazby pro DPH
		$eetSazby = array(0, 21, 15, 10);
		$sazbyNok = array_diff(array_keys($rozdeleni), $eetSazby);
		if (count($sazbyNok)>0) {
			$this->logger->writeLog("Neplatna sazba DPH " . implode(",", $sazbyNok) . ", polozky kosiku: " . print_r($orderToPayInfo->cartItems, true));
			return null;
		}


		// vypocet zakladu a dane
		$eetCastky = new EetCastkySazby();
		$eetCastky->celk_trzba = array_sum($rozdeleni);
		if (isset($rozdeleni[0])) {
			$eetCastky->zakl_nepodl_dph = $rozdeleni[0];
		}
		for ($i=1; $i<=3; $i++) {
			$sazba = $eetSazby[$i];
			if (isset($rozdeleni[$sazba])) {
				$celk = $rozdeleni[$sazba];
				if (empty($rozdeleniTax[$sazba])) {
					$koef = round($sazba/(100 + $sazba), 4);
					$dan = round($celk * $koef, 2);
				} else {
					$dan = $rozdeleniTax[$sazba];
				}
				$eetCastky->{"zakl_dan$i"} = $celk - $dan;
				$eetCastky->{"dan$i"} = $dan;
			}
		}
		return $eetCastky;
	}

	// vraci poznamku pro pridani k objednavce nebo null
	/**
	 * @param string $shopOrderNumber
	 * @param string $shopPairingInfo
	 * @param string $newShopOrderStatus
	 * @param ?callable() : OrderToPayInfo $funcGetOrderToPayInfo
	 * @return null | string | array{success: bool, message: string, newStatus?: string, result?: string}
	 */
	public function shopOrderStatusChanged($shopOrderNumber, $shopPairingInfo, $newShopOrderStatus, $funcGetOrderToPayInfo = null) {
		return null;
	}

	/**
	 * @param string $shopOrderNumber
	 * @param string $shopPairingInfo
	 * @return ?string
	 */
	public function getOrderAdminDetailHtml($shopOrderNumber, $shopPairingInfo) {
		return null;
	}


}
$vrfacke = "is_null";

class EetCastkySazby {
	/** @var number */
	var $celk_trzba;
	/** @var number */
	var $zakl_nepodl_dph;
	/** @var number */
	var $zakl_dan1;
	/** @var number */
	var $dan1;
	/** @var number */
	var $zakl_dan2;
	/** @var number */
	var $dan2;
	/** @var number */
	var $zakl_dan3;
	/** @var number */
	var $dan3;
}

class UniModulFactory extends CheckNonexistentFieldsLogOnly {
	/**
	 * @param string $name
	 * @param string $language
	 * @param ?string $subMethod
	 * @param string $adapterName
	 * @return ConfigInfo
	 */
	function getConfigInfo($name, $language='en', $subMethod=null, $adapterName=null) {
		/** @var ConfigSetting */
		$preCofingInfo = create_initialize_object('ConfigSetting', array('uniModulConfig'=>create_initialize_object('UniModulConfig', array('adapterName' => $adapterName))));
		$uniModul = $this->createUniModul($name, $preCofingInfo, $subMethod);
		$cofingInfo = $uniModul->getConfigInfo($language);

		$d = $uniModul->dictionary;
		$d->setDefaultLanguage($language);
		$configField = new ConfigField();
		$configField->name = 'activationKey';
		$configField->label = $d->get('activationKey');
		$configField->type = ConfigFieldType::$text;
		array_unshift($cofingInfo->configFields, $configField);

		return $cofingInfo;
	}

	/**
	 * @param string $name
	 * @param ConfigSetting $configSetting
	 * @param ?string $subMethod
	 * @return UniModul
	 */
	function createUniModul($name, $configSetting, $subMethod=null) {
		if (!ctype_alnum($name)) {
			throw new Exception('Neplatny UniModul: '.$name);
		}

		if ($configSetting->uniModulConfig->session === null) {
			$configSetting->uniModulConfig->session = new StandardSession();
		}

		$vniModul = "Uni".$name;
		require_once($vniModul.".php");
		/** @var UniModul */
		$unimod = new $vniModul($configSetting, $subMethod);
		return $unimod;
	}
}



class UniDictionary extends CheckNonexistentFieldsLogOnly {
	/** @var array<string, array<string, string>> */
	protected $dictionary;
	/** @var UniLogger */
	protected $logger;
	/** @var string */
	protected $dictionaryFile;

	/**
	 * @param string $dictionaryFile
	 * @param UniLogger $logger
	 * @return void
	 */
	function __construct($dictionaryFile, $logger) {
		/** @var array<string, array<string, string>> */
		$dict = array();
		include(dirname(__FILE__).'/Uni'."Modul"."Lang.php");  // nejak parametrizovat aby to bylo pripravene pro pluginy
		include(dirname(__FILE__).'/Uni'.$dictionaryFile."Lang.php");
		$this->dictionary = $dict;
		$this->dictionaryFile = $dictionaryFile;
		$this->logger = $logger;
		/* pro nacitani z CSV
		$fh = fopen(dirname(__FILE__).'/'.$dictionaryFile, "r");
			$langs = fgetcsv($fh, 1000, ";");
			$this->dictionary = array();
			for ($i=1; $i<count($langs); $i++) {
				$this->dictionary[$langs[$i]]=array();
			}
			while ($line = fgetcsv($fh, 1000, ";")) {
				$key = $line[0];
				for ($i=1; $i<count($line); $i++) {
					$this->dictionary[$langs[$i]][$key] = $line[$i];
				}
			}
		*/
	}

	/** @var string */
	private $defaultLanguage;

	/**
	 * @param string $language
	 * @return void
	 */
	function setDefaultLanguage($language) {
		$this->defaultLanguage = $language;
	}

	/**
	 * @param string $key
	 * @param string $language
	 * @return string
	 */
	function get($key, $language=null) {
		if ($language === null) {
			$language = $this->defaultLanguage;
		}
		if (isset($this->dictionary[$language]) && isset($this->dictionary[$language][$key])) {
			return $this->dictionary[$language][$key];
		} else {
			if (isset($this->dictionary['en'][$key])) {
				$r = $this->dictionary['en'][$key];
			} else {
				$r = $key;
			}
			//$this->logger->writeLog("WARN UniModul: Missing translation [$language:$key] in {$this->dictionaryFile} using '$r'");
			//user_error("WARN UniModul: Missing translation [$language:$key] in {$this->dictionaryFile} using $r");
			return $r;
		}
	}


}



class UniLogger extends CheckNonexistentFieldsLogOnly {
	/** @var string */
	var $logFile;

	function __construct() {
		$this->logFile = isset($GLOBALS['UniModul_Logfile']) ? $GLOBALS['UniModul_Logfile'] : dirname(__FILE__)."/logs/UniModul.log";
	}

	/**
	 * @param string $s
	 * @return void
	 */
	function writeLog($s) {
		$line = "*** ".date('r')." ".$s."\n";
		file_put_contents($this->logFile, $line, FILE_APPEND | LOCK_EX);
	}

	/**
	 * @param string $s
	 * @return void
	 */
	function writeLogNoNewLines($s) {
		$this->writeLog($this->replaceLogCrLf($s));
	}

	/**
	 * @param string $s
	 * @return string
	 */
	function replaceLogCrLf($s) {
		$s = str_replace("\r","",str_replace("\n","|",$s));
		return $s;
	}
}



/**
 * @template _T
 * @param class-string<_T> $className
 * @param array<string, mixed> $data
 * @return _T
 */
function create_initialize_object($className, $data) { // data array(n=>v)
	$object = new $className();
	foreach($data as $name => $value) {
		$object->{$name} = $value;
	}
	return $object;
}

/**
 * @param mixed $val
 * @return null|string
 */
function strvalNull($val) {
	return $val === null ? null : strval($val);
}


