<?php
// Autor (c) Miroslav Novak, www.platiti.cz
// Pouzivani bez souhlasu autora neni povoleno
// #Ver:PRV098-49-ga9cb64a5:2022-11-14#


//namespace OxidEsales\PayPalModule\Controller;

use OxidEsales\Eshop\Core\Registry;
use OxidEsales\EshopCommunity\Application\Model\Order;

require_once(Registry::getConfig()->getConfigParam('sShopDir') . '/UniModul/UniModul.php');
require_once(Registry::getConfig()->getConfigParam('sShopDir') . '/UniModul/OxidDbConnection.php');


class UniOrderControllerHelper
{
	protected $uniModulName;
	protected $helpedObject;
	protected $paymentId;

	public function __construct($helpedObject, $uniModul) {
		try {
			BeginUniErr();

			$this->helpedObject = $helpedObject;
			$this->uniModulName = $uniModul;
			$this->paymentId = strtolower($uniModul); // . 'binder';

			EndUniErr();
		} catch (Exception $ex) {
			ExceptionUniErr($ex);
		}
	}

    public function _getNextStep($iSuccess)
    {
		try {
			BeginUniErr();

			$lang = oxRegistry::getLang()->getLanguageAbbr();
			$oOrder = $this->getOrder();

			if ($this->isOurModule()) {
				if (is_numeric($iSuccess) && ($iSuccess == Order::ORDER_STATE_OK || $iSuccess == Order::ORDER_STATE_ORDEREXISTS)) {

					$uniModul = $this->createUniModul();
					$orderToPayInfo = $this->getOrderToPayInfo();

					$redirectAction = $uniModul->gatewayOrderRedirectAction($orderToPayInfo);

					if ($redirectAction->redirectForm != null) {
						$uniModul->formRedirect($redirectAction->redirectForm);
						die('redirecting...');
					} else if ($redirectAction->redirectUrl != null) {
						header('Location: '.$redirectAction->redirectUrl);
						ResetUniErr();
						exit();
					}
				}

			if ($iSuccess === 'GatewayReturn' && $oOrder) {

					$uniModul = $this->createUniModul();
					$orderReplyStatus = $uniModul->gatewayReceiveReply(oxRegistry::getLang()->getLanguageAbbr());
					if ($orderReplyStatus->orderStatus == OrderStatus::$successful) {
						$oOrder->oxorder__oxtransstatus = new oxField('OK');
						$oOrder->save();
						$oBasket = $this->helpedObject->getBasket();
						$oUser = $this->helpedObject->getUser();
						$oPayment = $this->helpedObject->getPayment();
						$pn = $this->uniModulName . "Parent_sendOrderByEmail";
						$iSuccess = $oOrder->$pn($oUser, $oBasket, $oPayment);
					} else {
						$oOrder->delete();
						$iSuccess = $orderReplyStatus->resultText;
					}

				}
			}
			$pn = $this->uniModulName . "Parent_getNextStep";
			return EndUniErr($this->helpedObject->$pn($iSuccess));
		} catch (Exception $ex) {
			ExceptionUniErr($ex);
		}
	}

	public function processGatewayReturn()
	{
		$iSuccess = 'GatewayReturn';
		return $this->_getNextStep($iSuccess);
	}

	private function getOrder()
	{

		$order = oxNew(\OxidEsales\Eshop\Application\Model\Order::class);
		$session = \OxidEsales\Eshop\Core\Registry::getSession();

		$order->load($session->getVariable('sess_challenge'));

		return $order;
	}

	private function isOurModule()
	{
		$session = \OxidEsales\Eshop\Core\Registry::getSession();
		return ($session->getVariable("paymentid") == $this->paymentId);
	}

	private function getOrderToPayInfo($subMethod=null, $possiblyCreateOrder=false) {
		$oOrder = $this->getOrder();

		$orderToPayInfo = new OrderToPayInfo();
		$orderToPayInfo->subMethod = $subMethod;
		$orderToPayInfo->shopOrderNumber = $oOrder->oxorder__oxordernr->value;
		$orderToPayInfo->shopPairingInfo = $oOrder->oxorder__oxordernr->value;
		$orderToPayInfo->amount = $oOrder->oxorder__oxtotalbrutsum->value;
		$orderToPayInfo->currency = $oOrder->oxorder__oxcurrency->value;
		$orderToPayInfo->language = oxRegistry::getLang()->getLanguageAbbr();
		$orderToPayInfo->description = $this->helpedObject->getConfig()->getActiveShop()->oxshops__oxname->value;

		$orderToPayInfo->replyUrl = $returnUrl = $this->helpedObject->getViewConfig()->getCurrentHomeDir()."index.php?lang=1&cl=order&fnc=processGatewayReturn";
		$orderToPayInfo->uniModulDirUrl = $this->helpedObject->getViewConfig()->getCurrentHomeDir()."/UniModul/";

		$oCountry = oxNew(\OxidEsales\Eshop\Application\Model\Country::class);
		$oCountry->load($oOrder->oxorder__oxbillcountryid->value);

		$customerData = new CustomerData();
		$customerData->first_name = $oOrder->oxorder__oxbillfname->value;
		$customerData->last_name = $oOrder->oxorder__oxbilllname->value;
		$customerData->email = $oOrder->oxorder__oxbillemail->value;
		$customerData->street = $oOrder->oxorder__oxbillstreet->value;
		$customerData->houseNumber = $oOrder->oxorder__oxbillstreetnr->value;
		$customerData->city = $oOrder->oxorder__oxbillcity->value;
		$customerData->post_code = $oOrder->oxorder__oxbillzip->value;
		$customerData->country = $oCountry->oxcountry__oxisoalpha2->value;
		$customerData->phone = $oOrder->oxorder__oxbillfon->value;
		$orderToPayInfo->customerData = $customerData;

		return $orderToPayInfo;
	}


	private function createUniModul() {
		$subMethod = '';
		$uniFact = new UniModulFactory();
		$configInfo = $uniFact->getConfigInfo($this->uniModulName, 'en', $subMethod);

		$configData = array();
		foreach ($configInfo->configFields as $configField) {
			if ($configField->type == ConfigFieldType::$text || $configField->type == ConfigFieldType::$choice) {
				$configData[$configField->name] = oxRegistry::getConfig()->getConfigParam($this->paymentId . "_" . $configField->name);
			} else {
				$configData[$configField->name] = null;
			}
		}

		$uniModulConfig = new UniModulConfig();
		$uniModulConfig->databaseConnection = new OxidDbConnection();
		$uniModulConfig->adapterName = 'Oxid';
		if ($this->helpedObject != null) {
			$uniModulConfig->uniModulDirUrl = $this->helpedObject->getViewConfig()->getCurrentHomeDir()."/UniModul/";
		}

		$configSetting = create_initialize_object('ConfigSetting', array('configData'=>$configData, 'uniModulConfig'=>$uniModulConfig));

		$uniModul = $uniFact->createUniModul($this->uniModulName, $configSetting, $subMethod);
		$uniModul->checkConfig($configSetting);
		$uniModul->dictionary->setDefaultLanguage(oxRegistry::getLang()->getLanguageAbbr());
		return $uniModul;
	}


	private function createUniModulStatic() {
		$uniFact = new UniModulFactory();
		$dummyConfigSettings = new ConfigSetting();
		$dummyConfigSettings->uniModulConfig = new UniModulConfig();
		$uniModul = $uniFact->createUniModul($this->uniModulName, $dummyConfigSettings, '');
		return array($uniModul, $uniFact);
	}

	public function getMetadata() {
		try {
			BeginUniErr();
			list($uniModul, $uniFact) = $this->createUniModulStatic();
			$configInfo = $uniFact->getConfigInfo($this->uniModulName, 'en', '');
			$settings = array();
			foreach ($configInfo->configFields as $configField) {
				if ($configField->type == ConfigFieldType::$text || $configField->type == ConfigFieldType::$choice) {
					$s = array(
						'group' => $this->paymentId . "_" . 'config',
						'name' => $this->paymentId . "_" . $configField->name
					);
					if ($configField->type == ConfigFieldType::$choice) {
						$s['type'] = 'select';
						$s['constraints'] = implode('|', array_keys($configField->choiceItems));
					} else {
						$s['type'] = 'str';
						$s['value'] = '';
					}
					$settings[] = $s;
				}
			}

			$sMetadataVersion = '1.1';

			$aModule = array(
				'id'          => $this->paymentId,
				'title'       => array(
					'de' => $uniModul->getModuleSubMethodName('de', ''),
					'en' => $uniModul->getModuleSubMethodName('en', ''),
					'cs' => $uniModul->getModuleSubMethodName('cs', ''),
					'sk' => $uniModul->getModuleSubMethodName('sk', ''),
					'hu' => $uniModul->getModuleSubMethodName('hu', ''),
					'ro' => $uniModul->getModuleSubMethodName('ro', ''),
				),
				'description' => array(
					'de' => $uniModul->getModuleSubMethodName('de', ''),
					'en' => $uniModul->getModuleSubMethodName('en', ''),
					'cs' => $uniModul->getModuleSubMethodName('cs', ''),
					'sk' => $uniModul->getModuleSubMethodName('sk', ''),
					'hu' => $uniModul->getModuleSubMethodName('hu', ''),
					'ro' => $uniModul->getModuleSubMethodName('ro', ''),
				),
				'thumbnail'   => '../../../UniModul/Uni'.$this->uniModulName.'Logo.png',
				'version'     => '1.0.0',
				'author'      => 'GPE',
				'url'         => 'https://gpe.cz/',
				'email'       => 'gpwebpay@gpe.com',

				'extend'      => array(
					'order' => 'pti/'.$this->paymentId.'/Controller/'.$this->uniModulName.'OrderController',
					'oxorder' => 'pti/'.$this->paymentId.'/Model/'.$this->uniModulName.'Order',
				),

				'files'       => array(
					$this->uniModulName.'Events'                 => 'pti/'.$this->paymentId.'/Core/'.$this->uniModulName.'Events.php',
				),

				'events'      => array(
					'onActivate'   => $this->uniModulName.'Events::onActivate',
					'onDeactivate' => $this->uniModulName.'Events::onDeactivate',
				),
				'settings' => $settings,
			);
			return EndUniErr(array($sMetadataVersion, $aModule));
		} catch (Exception $ex) {
			ExceptionUniErr($ex);
		}
	}

	public function getAdminTranslations($lang) {
		try {
			BeginUniErr();
			list($uniModul, $uniFact) = $this->createUniModulStatic();
			$configInfo = $uniFact->getConfigInfo($this->uniModulName, $lang, '');

			$sLangName = $lang;  // todo: nazev jazyka
			$aLang = array(
				'charset' => 'UTF-8',
				'SHOP_MODULE_GROUP_'.$this->paymentId.'_config' => 'Configuration',
			);

			foreach ($configInfo->configFields as $configField) {
				$fnt = 'SHOP_MODULE_'.$this->paymentId.'_'.$configField->name;
				if ($configField->type == ConfigFieldType::$text) {
					$aLang[$fnt] = $configField->label;
				}
				if ($configField->type == ConfigFieldType::$choice) {
					$aLang[$fnt] = $configField->label;
					foreach ($configField->choiceItems as $val=>$lab) {
						$aLang[$fnt.'_'.$val] = $lab;
					}
				}
			}
			return EndUniErr(array($sLangName, $aLang));
		} catch (Exception $ex) {
			ExceptionUniErr($ex);
		}
	}


    public function onActivate()
    {
		try {
			BeginUniErr();
			self::addPaymentMethod();
			self::enablePaymentMethod();
			self::addOrderTable();
			EndUniErr();
		} catch (Exception $ex) {
			ExceptionUniErr($ex);
		}
    }

    public function onDeactivate()
    {
		try {
			BeginUniErr();
			self::disablePaymentMethod();
			\OxidEsales\Eshop\Core\Registry::getSession()->delBasket();
			EndUniErr();
		} catch (Exception $ex) {
			ExceptionUniErr($ex);
		}
    }



    private function addPaymentMethod()
    {
		list($uniModul, $uniFact) = $this->createUniModulStatic();
        $payment = oxNew(\OxidEsales\Eshop\Application\Model\Payment::class);
        if (!$payment->load($this->paymentId)) {
            $payment->setId($this->paymentId);
            $payment->oxpayments__oxactive = new \OxidEsales\Eshop\Core\Field(1);
            $payment->oxpayments__oxdesc = new \OxidEsales\Eshop\Core\Field($uniModul->getModuleSubMethodName('en', ''));
            $payment->oxpayments__oxaddsum = new \OxidEsales\Eshop\Core\Field(0);
            $payment->oxpayments__oxaddsumtype = new \OxidEsales\Eshop\Core\Field('abs');
            $payment->oxpayments__oxfromboni = new \OxidEsales\Eshop\Core\Field(0);
            $payment->oxpayments__oxfromamount = new \OxidEsales\Eshop\Core\Field(0);
            $payment->oxpayments__oxtoamount = new \OxidEsales\Eshop\Core\Field(1000000);

            $language = \OxidEsales\Eshop\Core\Registry::getLang();
            $languages = $language->getLanguageIds();
            foreach (array('de', 'en', 'cs', 'sk', 'hu', 'ro') as $languageAbbreviation) {
                $languageId = array_search($languageAbbreviation, $languages);
                if ($languageId !== false) {
                    $payment->setLanguage($languageId);
                    $payment->oxpayments__oxlongdesc = new \OxidEsales\Eshop\Core\Field($uniModul->getModuleSubMethodName($languageAbbreviation, ''));
                    $payment->save();
                }
            }
        }
    }


    private function enablePaymentMethod()
    {
        $payment = oxNew(\OxidEsales\Eshop\Application\Model\Payment::class);
        $payment->load($this->paymentId);
        $payment->oxpayments__oxactive = new \OxidEsales\Eshop\Core\Field(1);
        $payment->save();
    }


    private function disablePaymentMethod()
    {
        $payment = oxNew(\OxidEsales\Eshop\Application\Model\Payment::class);
        if ($payment->load($this->paymentId)) {
            $payment->oxpayments__oxactive = new \OxidEsales\Eshop\Core\Field(0);
            $payment->save();
        }
    }


    private function addOrderTable()
    {
		$sql = file_get_contents (Registry::getConfig()->getConfigParam('sShopDir').'/UniModul/UniModul.sql');
        \OxidEsales\Eshop\Core\DatabaseProvider::getDb()->execute($sql);
    }

}
