<?php
// Autor (c) Miroslav Novak, www.platiti.cz
// Pouzivani bez souhlasu autora neni povoleno
// #Ver:PRV098-49-ga9cb64a5:2022-11-14#

/** @return void  */
function BeginSynchronized() {
	global $UniSyncFile;
	if (!isset($UniSyncFile)) {
		register_shutdown_function('SynchronizedShutdownHandler');
	}
	if ($UniSyncFile) {
		user_error("ERROR: BeginSynchronized v ramci aktivni synchronizace");
		return;
	}
	$UniSyncFile = fopen(dirname(__FILE__)."/sync.lock", "r+");
	if ($UniSyncFile === false) {
		user_error("ERROR: BeginSynchronized cannot open lock file");
		return;
	}
	if (!flock($UniSyncFile, LOCK_EX | LOCK_NB)) { // do an exclusive lock
		SynchronizedWriteLog("BLOCK: Cekam na sync");
		if(!flock($UniSyncFile, LOCK_EX)) {
			SynchronizedWriteLog("ERROR: Synchronizace se nezdarila - pokracuji bez synchronizace");
			user_error("ERROR: Synchronizace se nezdarila - pokracuji bez synchronizace");
			return;
		}
		SynchronizedWriteLog("BLOCK: Dockal jsem se");
	}
}

/** @return void  */
function EndSynchronized() {
	global $UniSyncFile;
	if ($UniSyncFile == null) {
		user_error("ERROR: EndSynchronized bez BeginSynchronized");
		return;
	}
	flock($UniSyncFile, LOCK_UN); // release the lock
	fclose($UniSyncFile);
	$UniSyncFile = 0;  // isset pak vraci true
}

/** @return void  */
function BeginMaybeExitSynchronized() {
	global $UniSyncFile, $UniSyncMaybeExit;
	if ($UniSyncFile == null) {
		user_error("ERROR: BeginMaybeExitSynchronized bez BeginSynchronized");
		return;
	}
	if ($UniSyncMaybeExit) {
		user_error("ERROR: BeginMaybeExitSynchronized jiz aktivni");
		return;
	}
	$UniSyncMaybeExit = true;
}

/** @return void  */
function EndMaybeExitSynchronized() {
	global $UniSyncFile, $UniSyncMaybeExit;
	if ($UniSyncFile == null) {
		user_error("ERROR: EndMaybeExitSynchronized bez BeginSynchronized");
		return;
	}
	if (!$UniSyncMaybeExit) {
		user_error("ERROR: EndMaybeExitSynchronized bez EndMaybeExitSynchronized");
		return;
	}
	$UniSyncMaybeExit = false;
}


/** @return void  */
function SynchronizedShutdownHandler() {
	global $UniSyncFile, $UniSyncMaybeExit;
	if ($UniSyncFile != null && !$UniSyncMaybeExit) {
		SynchronizedWriteLog("ERROR: Shutdown bez ukonceni synchronizace");
		user_error("ERROR: Shutdown bez ukonceni synchronizace");
	}
}


/**
 * @param string $msg
 * @return void
 */
function SynchronizedWriteLog($msg) {
	$logger = new UniLogger();
	$logger->WriteLog("Sync: ".$msg);
}
